/*
 * (c) 2012, Vyacheslav Grigoryev, <armag_vvg@hotmail.ru>
 *
 * This file is part of "SG WinReg Demo".
 *
 * "SG WinReg Demo" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * "SG WinReg Demo" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <windows.h>
#include <boost/format.hpp>

#include "misc.h"
#include "errors.h"


namespace sg
{

const std::string win_error::conv_last_error(const char *pMsg, DWORD dw)
{
	char *pMsgBuf = NULL;

	if(!::FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		dw,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
		(char*)&pMsgBuf,
		0, NULL))
	{
		if(pMsg)
			return (boost::format("%s: can't format message, raw code is 0x%0x") \
					% pMsg % dw).str();
		else
			return (boost::format("can't format message, raw code is 0x%0x") \
					% dw).str();
	}

	sg::scoped_handle<void*> buf(pMsgBuf, ::LocalFree);

	if(pMsg)
		return (boost::format("%s: (0x%0x) %s") % pMsg % dw % (char*)buf.get()).str();
	else
		return (boost::format("(0x%0x) %s") % dw % (char*)buf.get()).str();
}

} // end of namespace sg
